package com.fr.plugin.ymrt.sms.handle;

import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.data.User;
import com.fr.general.PropertiesUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.ymrt.sms.ResultModel;
import com.fr.plugin.ymrt.sms.bean.OutputSmsAction;
import com.fr.plugin.ymrt.sms.eucp.inter.http.v1.dto.request.SmsSingleRequest;
import com.fr.plugin.ymrt.sms.eucp.inter.http.v1.dto.response.SmsResponse;
import com.fr.plugin.ymrt.sms.util.AES;
import com.fr.plugin.ymrt.sms.util.GZIPUtils;
import com.fr.plugin.ymrt.sms.util.JsonHelper;
import com.fr.plugin.ymrt.sms.util.http.*;
import com.fr.schedule.base.constant.ScheduleConstants;
import com.fr.schedule.feature.output.OutputActionHandler;
import com.fr.stable.StringUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.restriction.RestrictionFactory;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author fr.open
 * @Date 2020/9/15
 * @Description
 **/
public class SmsOutputActionHandler extends OutputActionHandler<OutputSmsAction> {

    private String appId;

    private String secretKey;

    private String host;

    private String customSmsId;

    private String extendedCode;

    private final String algorithm = "AES/ECB/PKCS5Padding";
    // 编码
    private final String encode = "UTF-8";
    // 是否压缩
    private final boolean isGizp = true;


    public SmsOutputActionHandler() {
        String appid = PropertiesUtils.getProperties("ymrt").getProperty("appId");
        FineLoggerFactory.getLogger().info("get appid is {}", appid);
        this.appId = appid;
        String key = PropertiesUtils.getProperties("ymrt").getProperty("secretKey");
        FineLoggerFactory.getLogger().info("get key is {}", key);
        this.secretKey = key;
        String host = PropertiesUtils.getProperties("ymrt").getProperty("host");
        FineLoggerFactory.getLogger().info("get host is {}", host);
        this.host = host;
        String customSmsId = PropertiesUtils.getProperties("ymrt").getProperty("customSmsId");
        FineLoggerFactory.getLogger().info("get customSmsId is {}", customSmsId);
        this.customSmsId = customSmsId;
        String extendedCode = PropertiesUtils.getProperties("ymrt").getProperty("extendedCode");
        FineLoggerFactory.getLogger().info("get extendedCode is {}", extendedCode);
        this.extendedCode = extendedCode;
    }

    @Override
    public void doAction(OutputSmsAction action, Map<String, Object> map) throws Exception {
        FineLoggerFactory.getLogger().info("sam content is {}",action.getContent());
        String[] users = (String[]) map.get(ScheduleConstants.USERNAMES);
        String scheduleUsername = (String) map.get(ScheduleConstants.USERNAME);
        Set<String> userSet = new HashSet();
        if (StringUtils.isNotBlank(scheduleUsername)) {
            userSet.add(scheduleUsername);
        } else {
            userSet.addAll(Arrays.asList(users));
        }
        FineLoggerFactory.getLogger().info("config users is {}",userSet);
        List<User> userList = AuthorityContext.getInstance().getUserController().find(QueryFactory.create().addRestriction(RestrictionFactory.in("userName", userSet)));
        List<String> mobiles = userList.stream().filter(e -> StringUtils.isNotBlank(e.getMobile())).map(e -> e.getMobile()).collect(Collectors.toList());
        if(mobiles.isEmpty()){
            throw new Exception("config user mobile is null");
        }
        FineLoggerFactory.getLogger().info("send sms mobile is {}",mobiles);
        setSingleSms(appId,secretKey,host,algorithm,action.getContent(),customSmsId,extendedCode, com.fr.third.org.apache.commons.lang3.StringUtils.join(mobiles,","),isGizp,encode);
    }
    private static void setSingleSms(String appId, String secretKey, String host, String algorithm, String content, String customSmsId, String extendCode, String mobile, boolean isGzip, String encode) {
        FineLoggerFactory.getLogger().info("=============begin setSingleSms==================");
        SmsSingleRequest pamars = new SmsSingleRequest();
        pamars.setContent(content);
        pamars.setCustomSmsId(customSmsId);
        pamars.setExtendedCode(extendCode);
        pamars.setMobile(mobile);
        ResultModel result = request(appId, secretKey, algorithm, pamars, host + "/inter/sendSingleSMS", isGzip, encode);
        FineLoggerFactory.getLogger().info("result code :" + result.getCode());
        if ("SUCCESS".equals(result.getCode())) {
            SmsResponse response = JsonHelper.fromJson(SmsResponse.class, result.getResult());
            if (response != null) {
                FineLoggerFactory.getLogger().info("data : " + response.getMobile() + "," + response.getSmsId() + "," + response.getCustomSmsId());
            }
        }
        FineLoggerFactory.getLogger().info("=============end setSingleSms==================");
    }

    /**
     * 公共请求方法
     */
    public static ResultModel request(String appId, String secretKey, String algorithm, Object content, String url, final boolean isGzip, String encode) {
        Map<String, String> headers = new HashMap<String, String>();
        HttpRequest<byte[]> request = null;
        try {
            headers.put("appId", appId);
            headers.put("encode", encode);
            String requestJson = JsonHelper.toJsonString(content);
            FineLoggerFactory.getLogger().info("result json: " + requestJson);
            byte[] bytes = requestJson.getBytes(encode);
            FineLoggerFactory.getLogger().info("request data size : " + bytes.length);
            if (isGzip) {
                headers.put("gzip", "on");
                bytes = GZIPUtils.compress(bytes);
                FineLoggerFactory.getLogger().info("request data size [com]: " + bytes.length);
            }
            byte[] parambytes = AES.encrypt(bytes, secretKey.getBytes(), algorithm);
            FineLoggerFactory.getLogger().info("request data size [en] : " + parambytes.length);
            HttpRequestParams<byte[]> params = new HttpRequestParams<byte[]>();
            params.setCharSet("UTF-8");
            params.setMethod("POST");
            params.setHeaders(headers);
            params.setParams(parambytes);
            params.setUrl(url);
            if (url.startsWith("https://")) {
                request = new HttpsRequestBytes(params, null);
            } else {
                request = new HttpRequestBytes(params);
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(),e);
        }
        HttpClient client = new HttpClient();
        String code = null;
        String result = null;
        try {
            HttpResponseBytes res = client.service(request, new HttpResponseBytesPraser());
            if (res == null) {
                FineLoggerFactory.getLogger().info("请求接口异常");
                return new ResultModel(code, result);
            }
            if (res.getResultCode().equals(HttpResultCode.SUCCESS)) {
                if (res.getHttpCode() == 200) {
                    code = res.getHeaders().get("result");
                    if (code.equals("SUCCESS")) {
                        byte[] data = res.getResult();
                        FineLoggerFactory.getLogger().info("response data size [en and com] : " + data.length);
                        data = AES.decrypt(data, secretKey.getBytes(), algorithm);
                        if (isGzip) {
                            data = GZIPUtils.decompress(data);
                        }
                        FineLoggerFactory.getLogger().info("response data size : " + data.length);
                        result = new String(data, encode);
                        FineLoggerFactory.getLogger().info("response json: " + result);
                    }
                } else {
                    FineLoggerFactory.getLogger().info("请求接口异常,请求码:" + res.getHttpCode());
                }
            } else {
                FineLoggerFactory.getLogger().info("请求接口网络异常:" + res.getResultCode().getCode());
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(),e);
        }
        ResultModel re = new ResultModel(code, result);
        return re;
    }
}